//*****************************************
// AT mega328P-AU(32pin TQFP) four z573m clock Program
// CPU Clock:8MHz
//   iq[Yf[^@WȂBIV[^(8MHz)ɂj
//
// File Name:nixie.c
//
//  2017/09/13 New TAB4 Taishi
//  2017/12/13 Ver 2.00 Taishi
//  2018/10/25 Modify for Fifteen IN-17 Clock Taishi
//  2019/12/30 Modify for fourteen IN-172 Clock Taishi
//  2020/09/17 Modify for six IN-16 clock Taishi
//  2020/09/24 Modify for four IN-8 clock Taishi
//  2020/12/23 Modify for four z573m clock Taishi
//*****************************************

#include	<avr/pgmspace.h>
#include	<avr/io.h>

#include	"four_z573m_clock.h"
#include	"nixie.h"

// **********************************************************
// *********************** NIXE TUBE ************************
// **********************************************************
//

// **********************************************************
//@jLV[ǂ͂O`RARł͂Ȃ@jLV[Ǔ̃hbgg
//@@jLV[ǑÍ@obO`R̃rbgɂđI
//@@@R@Q@P@O
//@@@k@k@k@k@||@O@@̏㌅
//@@@k@k@k@g@||@P@@̉
//@@@k@k@g@k@||@Q@@̏㌅
//@@@k@k@g@g@||@R@@̉
//
//@@@iAj
//@@\locSCocTCocUCocV̂Srbĝabcōs
//@@@ocV@ocU@ocT@ocS
//@@@k@@@k@@@k@@@k@@||@O
//@@@k@@@k@@@k@@@g@@||@P
//@@@k@@@k@@@g@@@k@@||@Q
//@@@k@@@k@@@g@@@g@@||@R
//@@@k@@@g@@@k@@@k@@||@S
//@@@k@@@g@@@k@@@g@@||@T
//@@@k@@@g@@@g@@@k@@||@U
//@@@k@@@g@@@g@@@g@@||@V
//@@@g@@@k@@@k@@@k@@||@W
//@@@g@@@k@@@k@@@g@@||@X
//@@@g@@@k@@@g@@@k@@||@
//@@@g@@@k@@@g@@@g@@||@
//@@@g@@@g@@@k@@@k@@||@
//@@@g@@@g@@@k@@@g@@||@
//@@@g@@@g@@@g@@@k@@||@
//@@@g@@@g@@@g@@@g@@||@
//
//@@hbg͂ocPŐ䂵Ae̐\Ɠ^C~Oŏo͂B
//@@yTVRliEhbgj킸@yTVSlihbgjgĂ邪
//@@Ehbg̃słhbg̃sł 悤ɏ̃hbg_̂
//@@hbg̐Mg悤ɂB
//@@ŏʂ z573m z570m z574m z574m ̕тɂȂĂBiz570m ̓hbgj
// **********************************************************

unsigned char	path_limit = 1;
unsigned char	change_path = 2;

static	unsigned char	nixie_data[KETA_MAX+2];			// +1 ŗǂAÔ
static	unsigned char	nixie_dpr[KETA_MAX+2];			// +1 ŗǂAÔ
static	unsigned char	nixie_dpl[KETA_MAX+2];			// +1 ŗǂAÔ
static	unsigned char	nixie_select[KETA_MAX+2];		// +1 ŗǂAÔ
static	unsigned char	nixie_act_num = 0;
static	unsigned char	dynamic_nixie = 0;


#define		NIXE_SELECT_DIS		0b11000000	// PC0 -- 5 = all 0
#define		NIXE_NUMBER_OFF		0b11110000	// PD4--7:'H'
#define		NIXE_NUMBER_MASK	0b00001111	// PD4--7:'L'ɂ鎞
#define		NIXE_DPR_ON			0b00000001	// PB0:'H'
#define		NIXE_DPL_ON			0b00000010	// PD1:'H'
#define		NIXE_DPR_ON			0b00000001	// PB0:'H'
//******************************
//  Nixe\@
// *****************************
void	init_nixe(void)
{
	unsigned char	uc;

	PORTC &= NIXE_SELECT_DIS;					// IȂ


	PORTD |= NIXE_NUMBER_OFF;					// Ȃ
	PORTB &= ~(NIXE_DPR_ON);					// q|cnsȂ
	PORTD &= ~(NIXE_DPL_ON);					// k|cnsȂ

// o̓|[g̐ݒigȂ ͂ƃmCŶŁ@o͂ɂ
	DDRD |= ((1 << DDD1) | (1 << DDD4) | (1 << DDD5) | (1 << DDD6) | (1 << DDD7));	// Port-D Bit1,4567 : Output
	DDRC |= ((1 << DDC0) | (1 << DDC1) | (1 << DDC2) | (1 << DDC3) | (1 << DDC4) | (1 << DDC5));	// Port-C Bit012345 : Output
	DDRB |= (1 << DDB0);						// Port-B Bit01 : Output

// \Ahbg f[^
	for(uc = 0;uc <= KETA_MAX; uc++){
		nixie_data[uc] = NIXE_NUMBER_OFF;		// NA
		nixie_dpr[uc] = 0;						// R-dot : OFF
		nixie_dpl[uc] = 0;						// L-dot : OFF
	}

	nixie_select[0] = 0b00000001;		// ŏʁ@̏㌅
	nixie_select[1] = 0b00000010;		// Qʁ@̉
	nixie_select[2] = 0b00000100;		// Rʁ@̏㌅
	nixie_select[3] = 0b00001000;		// ŉʁ@b̉

	nixie_act_num = 0;

	dynamic_nixie = 0;
}
//******************************
// jLV[Ǖ\
// sQ̊荞݂ŌĂ΂
//  for four z573m Clock
//******************************
void	proc_dynamic_nixie(void)
{
	static	unsigned char	path=0;

	if(dynamic_nixie == 1){
		if(path == path_limit){

			PORTC &= NIXE_SELECT_DIS;					// IȂ

			PORTD |= NIXE_NUMBER_OFF;					// Ȃ
			PORTB &= ~(NIXE_DPR_ON);					// NON Dot-R
			PORTD &= ~(NIXE_DPL_ON);					// NON Dot-L
			if(nixie_act_num <= KETA_MAX){				// ijLV[ǁj
				PORTD &= NIXE_NUMBER_MASK;				// }XN
				PORTD |= nixie_data[nixie_act_num];		// 
				PORTB |= nixie_dpr[nixie_act_num];		// Dot-R
				PORTD |= nixie_dpl[nixie_act_num];		// Dot-L
				PORTC |= nixie_select[nixie_act_num];	// I
			}

			if(KETA_MAX <= nixie_act_num){
				nixie_act_num = 0;
			}else{
				nixie_act_num++;
			}
		}else if(path == 0){
			PORTC &= NIXE_SELECT_DIS;					// IȂ

			PORTD |= NIXE_NUMBER_OFF;					// Ȃ
			PORTB &= ~(NIXE_DPR_ON);					// NON Dot-R
			PORTD &= ~(NIXE_DPL_ON);					// NON Dot-L
		}

		path++;
		if(PATH_MAX < path){
			path = 0;
		}

	}
}

//******************************
// Dynamic Nixie display Active/Non active
//  on : 1= Active, other=Non Active
// *****************************
void	dynamic_nixie_active(unsigned char on)
{
	dynamic_nixie = on;
}
//******************************
// Nixie \i_Ci~bN_j
//  k : ʒuiO`TAOFŏʁEEETFŉʁj
//  n : O`XAPTiPT͋󔒁j
// *****************************
void	disp_dynamic_nixie_number(unsigned char k, unsigned char n)
{
	if(KETA_MAX < k){						// IlُȎ
		k = 0;								// Iɍŏʂɂ
	}

	dynamic_nixie_active(0);				// _Ci~bN\~
	nixie_data[k] = ((n << 4) & 0xf0);
	dynamic_nixie_active(1);				// _Ci~bN\ĊJ
}

// *****************************
// Nixie c|k\i_Ci~bN_j
//  k : ʒuiO`TAOFŏʁEEETFŉʁj
//  n : O(OFF) or 1(ON)
//   _Ci~bN\͍sĂȂԂɌ
// *****************************
void	disp_dynamic_dotl_on_off(unsigned char k, unsigned char n)
{
	if(KETA_MAX < k){					// IlُȎ
		k = 0;							// Iɍŏʂɂ
	}
	dynamic_nixie_active(0);			// _Ci~bN\~
	if(n == 0){
		nixie_dpl[k] = 0;
	}else{
		nixie_dpl[k] = NIXE_DPL_ON;
	}
	dynamic_nixie_active(1);			// _Ci~bN\ĊJ
}
// *****************************
// Nixie c|q\i_Ci~bN_j
//  k : ʒuiO`TAOFŏʁEEETFŉʁj
//  n : O(OFF) or 1(ON)
//   _Ci~bN\͍sĂȂԂɌ
// *****************************
void	disp_dynamic_dotr_on_off(unsigned char k, unsigned char n)
{
	if(KETA_MAX < k){					// IlُȎ
		k = 0;							// Iɍŏʂɂ
	}
	dynamic_nixie_active(0);			// _Ci~bN\~
	if(n == 0){
		nixie_dpr[k] = 0;
	}else{
		nixie_dpr[k] = NIXE_DPR_ON;
	}
	dynamic_nixie_active(1);			// _Ci~bN\ĊJ
}

//******************************
// Nixie \SAci_Ci~bN_j
// *****************************
void	disp_dynamic_nixie_clear(void)
{
	unsigned char uc;

	for(uc=0;uc <= KETA_MAX; uc++){
		disp_dynamic_nixie_number(uc, 15);		//   
		disp_dynamic_dotl_on_off(uc, 0);		// c|k\ nee
		disp_dynamic_dotr_on_off(uc, 0);		// c|q\ nee
	}
}

//******************************
// Nixie \iX^eBbNN_j
//  k : ʒuiO`TAOFŏʁEEETFŉʁj
//  n : O`XAPTiPT͋󔒁j
//   _Ci~bN\͍sĂȂԂɌ
// *****************************
void	disp_static_nixie_number(unsigned char k, unsigned char n)
{
	PORTC &= NIXE_SELECT_DIS;			// IȂ

	PORTD |= NIXE_NUMBER_OFF;			// Ȃ
	PORTB &= ~(NIXE_DPR_ON);			// NON Dot-R
	PORTD &= ~(NIXE_DPL_ON);			// NON Dot-L

	if(KETA_MAX < k){					// IlُȎ
		k = 0;							// Iɍŏʂɂ
	}

	PORTD &= NIXE_NUMBER_MASK;			// }XN
	PORTD |= ((n << 4) & 0xf0);			// 

	PORTC |= nixie_select[k];			// I


}
//******************************
// Nixie c|q\iX^eBbNN_j
//  k : ʒuiO`TAOFŏʁEEETFŉʁj
//  n : O(OFF) or 1(ON)
//   _Ci~bN\͍sĂȂԂɌ
// *****************************
void	disp_static_dotr_on_off(unsigned char k, unsigned char n)
{
	PORTC &= NIXE_SELECT_DIS;			// IȂ

	PORTD |= NIXE_NUMBER_OFF;			// Ȃ
	PORTB &= ~(NIXE_DPR_ON);			// OFF Dot-R
	PORTD &= ~(NIXE_DPL_ON);			// OFF Dot-L

	if(KETA_MAX < k){					// IlُȎ
		k = 0;							// Iɍŏʂɂ
	}

	if(n != 0){
		PORTB |= (NIXE_DPR_ON);			// Dot-R : ON
	}

	PORTC |= nixie_select[k];			// I

}
//******************************
// Nixie c|k\iX^eBbNN_j
//  k : ʒuiO`TAOFŏʁEEETFŉʁj
//  n : O(OFF) or 1(ON)
//   _Ci~bN\͍sĂȂԂɌ
// *****************************
void	disp_static_dotl_on_off(unsigned char k, unsigned char n)
{
	PORTC &= NIXE_SELECT_DIS;			// IȂ

	PORTD |= NIXE_NUMBER_OFF;			// Ȃ
	PORTB &= ~(NIXE_DPR_ON);			// OFF Dot-R
	PORTD &= ~(NIXE_DPL_ON);			// OFF Dot-L

	if(KETA_MAX < k){					// IlُȎ
		k = 0;							// Iɍŏʂɂ
	}

	if(n != 0){
		PORTD |= (NIXE_DPL_ON);			// Dot-L : ON
	}

	PORTC |= nixie_select[k];			// I

}
